==> rospell-2007.01.rc3.tar.gz: rospell-2007.01.rc3/src/rospell/briefedit.cpp <==
//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "briefedit.h"
#include "qwerty.h"

static char *full_line_err = "Full line selection required!";

BriefEdit::BriefEdit(Editor *editor, QWidget * parent, const char * name): Q3MultiLineEdit(parent, name) {
	editor_ = editor;
	normal_insert_ = true;
	recording_ = 0;
	alias_buf_ = "";
	alist_ = Global::instance()->alist;
}


void BriefEdit::selectLine(int para, int index) {
	getCursorPosition(&para, &index);
	QString line = textLine(para);
	int len = line.length();
	int lines = numLines();
	if (para == (lines - 1)) { 
		setSelection(para, 0, para, len);
	}
	else {
		setSelection(para, 0, para + 1, 0);
	}

	normal_insert_ = false;
}

void BriefEdit::keyPressEvent(QKeyEvent *e) {
	LOG("BriefEdit::keyPressEvent: %x/%x", e->state(), e->key());
	if (recording_) {
		list_.push_back(*e);
	}
	int para;
	int index;
	getCursorPosition(&para, &index);
	
	// numeric pad + and - keys
	if ((unsigned) e->state() & Qt::Keypad) {
		normal_insert_ = true;

		if (e->key() == Qt::Key_Plus) {
			LOG("key numpad plus");
			if (!hasMarkedText())
				selectLine(para, index);
			copy();
			deselect();
			setCursorPosition(para, index, false);
			return;
		}
		else if (e->key() == Qt::Key_Minus) {
			LOG("key numpad minus");
			if (!hasMarkedText())
				selectLine(para, index);
			cut();
			setCursorPosition(para, index, false);
			return;
		}
	}

	// insert key
	if (e->key() == Qt::Key_Insert) {
		if (!normal_insert_) {
			setCursorPosition(para, 0, false);
		}
		paste();
		return;
	}

	// enter key
	if (e->key() == Qt::Key_Return) {
		insert("\n");
		if (para != 0) {
			QString tmp = textLine(para);
			int len = tmp.length();
			for (int i = 0; i < len; i++) {
				if (tmp.at(i) == ' ' || tmp.at(i) == '\t')
					insert(QString(tmp.at(i)));
				else
					break;
			}

		}
		return;
	}
	
	// romanian special characters
	if (e->state() & Qt::AltButton) {
		QString str;
		bool done = false;

		if (e->key() == Qt::Key_A) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ă");
			else
				str = QString::fromUtf8("ă");
			done = true;
		}
		else if (e->key() == Qt::Key_S) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ş");
			else
				str = QString::fromUtf8("ş");
			done = true;
		}
		else if (e->key() == Qt::Key_Q) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Â");
			else
				str = QString::fromUtf8("â");
			done = true;
		}
		else if (e->key() == Qt::Key_T) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ţ");
			else
				str = QString::fromUtf8("ţ");
			done = true;
		}
		else if (e->key() == Qt::Key_I) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Î");
			else
				str = QString::fromUtf8("î");
			done = true;
		}

		if (done) {
			insertAt(str, para, index, false);
			setCursorPosition(para, index + 1);
			return;
		}
	}

	// check aliases
	if (e->key() == Qt::Key_Shift); // this could be a shift key pressed!!!
	else if (e->key() >= 0x80)
		alias_buf_ = "";
	else
		alias_buf_.prepend((char) e->key());
	alias_buf_.truncate(8);

	if (alias_buf_.startsWith(" ")) {
		list<Global::Alias>::iterator it;
		for (it = alist_.begin(); it != alist_.end(); ++it) {
			Global::Alias al = *it;
			QString s = alias_buf_.mid(1);
			if (s.startsWith(al.reverse)) {
				// delete
				setSelection(para, index - al.alias.length(), para, index);
				del();
				// and insert replacement here
				getCursorPosition(&para, &index);
				insertAt(al.replace, para, index, false);
				setCursorPosition(para, index + al.replace.length());
				alias_buf_ = "";
				return;
			}
		}
	}	
//	LOG("alias_buf_: %s", (const char *) alias_buf_);
	return Q3MultiLineEdit::keyPressEvent(e);
}


bool BriefEdit::recordAction() {
	if (recording_ == 0) {
		//clear the list here
		while (!list_.empty())
			list_.erase(list_.begin());
		recording_ = 1;
		return true;
	}

	recording_ = 0;
	return false;
}

void BriefEdit::playbackAction() {
	if (recording_)
		recording_ = 0;

	list<QKeyEvent>::iterator it;
	for (it = list_.begin(); it != list_.end(); ++it) {
		QKeyEvent ev = *it;
		keyPressEvent(&ev);
	}
}

void BriefEdit::reloadAlias() {
	alist_ = Global::instance()->alist;
	LOG("reload alias");
}

QPopupMenu *BriefEdit::createPopupMenu(const QPoint& pos) {
	QPopupMenu *menu = Q3MultiLineEdit::createPopupMenu( pos );
	int id = menu->insertItem("Shift Left");
	menu->connectItem(id, this, SLOT(shiftLeftClicked(int)));
	id = menu->insertItem("Shift Right");
	menu->connectItem(id, this, SLOT(shiftRightClicked(int)));
	id = menu->insertItem("Space to Tab");
	menu->connectItem(id, this, SLOT(space2tabClicked(int)));
	id = menu->insertItem("Tab to Space");
	menu->connectItem(id, this, SLOT(tab2spaceClicked(int)));
	menu->insertSeparator();
	id = menu->insertItem("Quick Search");
	menu->connectItem(id, this, SLOT(quickSearchClicked(int)));

	return menu;
}

void BriefEdit::quickSearchClicked(int id) {
	(void) id;
	int para;
	int pos;
	getCursorPosition(&para, &pos);

	// lucian: some code here is duplicated from qwerty.cpp void Editor::cursorPosChanged(int para, int pos) - to fix!

	QString line = textLine(para);
	QChar ch = line.at(pos);
	if (ch.isLetterOrNumber() || ch == '_') {
		// extract word
		int xstart = pos;
		while (xstart >= 0 && (line.at(xstart).isLetterOrNumber() || line.at(xstart) == '_'))
			xstart--;
		if (xstart != 0)
			xstart++;

		int xend = pos;
		while (line.at(xend).isLetterOrNumber() || line.at(xend) == '_')
			xend++;

		QString ctagword = line.mid(xstart, xend - xstart);
		if (ctagword.startsWith("\t")) // lucian: some problems with the first letter when is at the beginning of the line! Below is just a hack!
			ctagword = ctagword.mid(1);
		if (ctagword.startsWith("\\"))
			ctagword = ctagword.mid(1);
		
		if (editor_->advanceSearch(ctagword, false))
			editor_->advanceSearchList(ctagword, false);
	}
}

void BriefEdit::shiftLeftClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}
	
	QString newstr = "";	
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		if (*ptr == ' ' || *ptr == '\t')
			newstr += ptr + 1; // delete the first character
		else 
			newstr += ptr;
		newstr += '\n';
	}
			
	// delete the old selection and insert the new text
	insert(newstr);
	
	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::shiftRightClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}
	
	QString newstr = "";	
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		newstr += '\t';
		newstr += line;
		newstr += '\n';
	}
			
	// delete the old selection and insert the new text
	insert(newstr);
	
	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::space2tabClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}
	
	QString newstr = "";	
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		while(*ptr == ' ') {
			newstr += '\t';
			ptr++;
		}
		newstr += ptr;
		newstr += '\n';
	}
			
	// delete the old selection and insert the new text
	insert(newstr);
	
	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::tab2spaceClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}
	
	QString newstr = "";	
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		while(*ptr == '\t') {
			newstr += ' ';
			ptr++;
		}
		newstr += ptr;
		newstr += '\n';
	}
			
	// delete the old selection and insert the new text
	insert(newstr);
	
	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

==> rospell-2007.02.rc1.tar.gz: rospell-2007.02.rc1/src/rospell/briefedit.cpp <==
//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "briefedit.h"
#include "qwerty.h"

static char *full_line_err = "Full line selection required!";

BriefEdit::BriefEdit(Editor *editor, QWidget * parent, const char * name): Q3MultiLineEdit(parent, name) {
	editor_ = editor;
	normal_insert_ = true;
	recording_ = 0;
	alias_buf_ = "";
	alist_ = Global::instance()->alist;
}


void BriefEdit::selectLine(int para, int index) {
	getCursorPosition(&para, &index);
	QString line = textLine(para);
	int len = line.length();
	int lines = numLines();
	if (para == (lines - 1)) {
		setSelection(para, 0, para, len);
	}
	else {
		setSelection(para, 0, para + 1, 0);
	}

	normal_insert_ = false;
}

void BriefEdit::keyPressEvent(QKeyEvent *e) {
	LOG("BriefEdit::keyPressEvent: %x/%x", e->state(), e->key());
	if (recording_) {
		list_.push_back(*e);
	}
	int para;
	int index;
	getCursorPosition(&para, &index);
	int key = e->key();

	// numeric pad + and - keys
	if ((unsigned) e->state() & Qt::Keypad) {
		normal_insert_ = true;

		if (key == Qt::Key_Plus) {
			LOG("key numpad plus");
			if (!hasMarkedText())
				selectLine(para, index);
			copy();
			deselect();
			setCursorPosition(para, index, false);
			return;
		}
		else if (key == Qt::Key_Minus) {
			LOG("key numpad minus");
			if (!hasMarkedText())
				selectLine(para, index);
			cut();
			setCursorPosition(para, index, false);
			return;
		}
	}

	// insert key
	if (key == Qt::Key_Insert) {
		if (!normal_insert_) {
			setCursorPosition(para, 0, false);
		}
		paste();
		return;
	}

	// enter key
	if (key == Qt::Key_Return) {
		insert("\n");
		if (para != 0) {
			QString tmp = textLine(para);
			int len = tmp.length();
			for (int i = 0; i < len; i++) {
				if (tmp.at(i) == ' ' || tmp.at(i) == '\t')
					insert(QString(tmp.at(i)));
				else
					break;
			}

		}
		return;
	}

	// romanian special characters
	Global *glb = Global::instance();
	if (e->state() & Qt::AltButton) {
		QString str;
		bool done = false;

		if (key == Qt::Key_A) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ă");
			else
				str = QString::fromUtf8("ă");
			done = true;
		}
		else if (key == Qt::Key_S && glb->lang_ro_commabelow) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ș");
			else
				str = QString::fromUtf8("ș");
			done = true;
		}
		else if (key == Qt::Key_S) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ş");
			else
				str = QString::fromUtf8("ş");
			done = true;
		}
		else if (key == Qt::Key_Q) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Â");
			else
				str = QString::fromUtf8("â");
			done = true;
		}
		else if (key == Qt::Key_T && glb->lang_ro_commabelow) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ț");
			else
				str = QString::fromUtf8("ț");
			done = true;
		}
		else if (key == Qt::Key_T) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ţ");
			else
				str = QString::fromUtf8("ţ");
			done = true;
		}
		else if (key == Qt::Key_I) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Î");
			else
				str = QString::fromUtf8("î");
			done = true;
		}

		if (done) {
			insertAt(str, para, index, false);
			setCursorPosition(para, index + 1);
			return;
		}
	}

	// check aliases
	if (key == Qt::Key_Shift); // this could be a shift key pressed!!!
	else if (key >= 0x80)
		alias_buf_ = "";
	else
		alias_buf_.prepend((char) key);
	alias_buf_.truncate(8);

	if (alias_buf_.startsWith(" ")) {
		list<Global::Alias>::iterator it;
		for (it = alist_.begin(); it != alist_.end(); ++it) {
			Global::Alias al = *it;
			QString s = alias_buf_.mid(1);
			if (s.startsWith(al.reverse)) {
				// delete
				setSelection(para, index - al.alias.length(), para, index);
				del();
				// and insert replacement here
				getCursorPosition(&para, &index);
				insertAt(al.replace, para, index, false);
				setCursorPosition(para, index + al.replace.length());
				alias_buf_ = "";
				return;
			}
		}
	}
//	LOG("alias_buf_: %s", (const char *) alias_buf_);
	return Q3MultiLineEdit::keyPressEvent(e);
}

bool BriefEdit::recordAction() {
	if (recording_ == 0) {
		//clear the list here
		while (!list_.empty())
			list_.erase(list_.begin());
		recording_ = 1;
		return true;
	}

	recording_ = 0;
	return false;
}

void BriefEdit::playbackAction() {
	if (recording_)
		recording_ = 0;

	list<QKeyEvent>::iterator it;
	for (it = list_.begin(); it != list_.end(); ++it) {
		QKeyEvent ev = *it;
		keyPressEvent(&ev);
	}
}

void BriefEdit::reloadAlias() {
	alist_ = Global::instance()->alist;
	LOG("reload alias");
}

QPopupMenu *BriefEdit::createPopupMenu(const QPoint& pos) {
	QPopupMenu *menu = Q3MultiLineEdit::createPopupMenu( pos );
	int id = menu->insertItem("Shift Left");
	menu->connectItem(id, this, SLOT(shiftLeftClicked(int)));
	id = menu->insertItem("Shift Right");
	menu->connectItem(id, this, SLOT(shiftRightClicked(int)));
	id = menu->insertItem("Space to Tab");
	menu->connectItem(id, this, SLOT(space2tabClicked(int)));
	id = menu->insertItem("Tab to Space");
	menu->connectItem(id, this, SLOT(tab2spaceClicked(int)));
	id = menu->insertItem("Sort Lines");
	menu->connectItem(id, this, SLOT(sortLinesClicked(int)));
	menu->insertSeparator();
	id = menu->insertItem("Quick Search");
	menu->connectItem(id, this, SLOT(quickSearchClicked(int)));

	return menu;
}

void BriefEdit::quickSearchClicked(int id) {
	(void) id;
	int para;
	int pos;
	getCursorPosition(&para, &pos);

	// lucian: some code here is duplicated from qwerty.cpp void Editor::cursorPosChanged(int para, int pos) - to fix!

	QString line = textLine(para);
	QChar ch = line.at(pos);
	if (ch.isLetterOrNumber() || ch == '_') {
		// extract word
		int xstart = pos;
		while (xstart >= 0 && (line.at(xstart).isLetterOrNumber() || line.at(xstart) == '_'))
			xstart--;
		if (xstart != 0)
			xstart++;

		int xend = pos;
		while (line.at(xend).isLetterOrNumber() || line.at(xend) == '_')
			xend++;

		QString ctagword = line.mid(xstart, xend - xstart);
		if (ctagword.startsWith("\t")) // lucian: some problems with the first letter when is at the beginning of the line! Below is just a hack!
			ctagword = ctagword.mid(1);
		if (ctagword.startsWith("\\"))
			ctagword = ctagword.mid(1);

		if (editor_->advanceSearch(ctagword, false))
			editor_->advanceSearchList(ctagword, false);
	}
}

void BriefEdit::shiftLeftClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		if (*ptr == ' ' || *ptr == '\t')
			newstr += ptr + 1; // delete the first character
		else
			newstr += ptr;
		newstr += '\n';
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::shiftRightClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		newstr += '\t';
		newstr += line;
		newstr += '\n';
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::space2tabClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		while(*ptr == ' ') {
			newstr += '\t';
			ptr++;
		}
		newstr += ptr;
		newstr += '\n';
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::tab2spaceClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		while(*ptr == '\t') {
			newstr += ' ';
			ptr++;
		}
		newstr += ptr;
		newstr += '\n';
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::sortLinesClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	// build the list
	list<QString> lst;
	for (int i = para_from; i < para_to; i++) {
		lst.push_back(textLine(i));
	}
	// sort
	lst.sort();
	// put the list back
	QString newstr = "";
	list<QString>::iterator it;
	for (it = lst.begin(); it != lst.end(); ++it) {
		QString s = *it;
		newstr += s + "\n";
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}
